@@ -200,7 +200,15 @@ pub struct ResolvedNetworkingProfile<R: Resolver = AsyncHyperResolver> {
 
 impl<R: Resolver> ResolvedNetworkingProfile<R> {
 	fn new(p: NetworkingProfile<R>) -> Result<Self> {
-		let resolver = if let Some(r) = p.resolver { r } else { Arc::new(R::new_default().context("cannot create default resolver")?.with_net_blacklist(Arc::clone(&RESERVED_SUBNETS))) };
+		let resolver = if let Some(r) = p.resolver {
+			r
+		} else {
+			Arc::new(
+				R::new_default()
+					.context("cannot create default resolver")?
+					.with_net_blacklist(Arc::clone(&RESERVED_SUBNETS)),
+			)
+		};
 
 		Ok(Self { values: p.values, resolver })
 	}
@@ -241,7 +249,7 @@ impl Default for CrawlingSettings {
 			job_hard_timeout:          CDuration::from_secs(60),
 			job_soft_timeout:          CDuration::from_secs(30),
 			load_timeout:              CDuration::from_secs(10),
-			user_agent:                Some(String::from("crusty-core/0.12.0")),
+			user_agent:                Some(String::from("crusty-core/0.13.0")),
 			compression:               true,
 			custom_headers:            HashMap::new(),
 			max_response_size:         CBytes(1024 * 1024 * 2),
